@@ -2,8 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Massachusetts
-    Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -28,7 +28,7 @@ USA.
 
 (declare (usual-integrations))
 
-(define hash-of-false (object-hash #f))
+(define hash-of-false (hash-object #f))
 (define match-group hash-of-false)
 
 (define (re-match-start i)
@@ -48,13 +48,13 @@ USA.
 			(re-match-end-index i)))
 
 (define (re-match-group)
-  (let ((group (object-unhash match-group)))
+  (let ((group (unhash-object match-group)))
     (if (not group)
 	(error "No match group"))
     group))
 
 (define (re-match-data)
-  (let ((group (object-unhash match-group)))
+  (let ((group (unhash-object match-group)))
     (cons group
 	  (if group
 	      (let ((v (make-vector 20)))
